﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Web;
using System.Reflection;
using System.IO;
using System.Text.RegularExpressions;

using Umc.Core.Configuration;

namespace Umc.Core.Web.HttpCompression
{
	/// <summary>
	/// Umc.Core<br/>
	/// HTTP 압축을 수행하는 HTTP Module 클래스 입니다.
	/// </summary>
	public sealed class HttpCompressionModule : IHttpModule
	{
		#region IHttpModule 멤버

		public void Dispose()
		{
		}

		public void Init(HttpApplication context)
		{
			context.BeginRequest += new EventHandler(context_BeginRequest);
			context.PreRequestHandlerExecute += new EventHandler(context_PreRequestHandlerExecute);
			context.PostRequestHandlerExecute += new EventHandler(context_PostRequestHandlerExecute);
		}

		void context_PostReleaseRequestState(object sender, EventArgs e)
		{
			HttpApplication app		= sender as HttpApplication;
			HttpContext context		= app.Context;
		}

		#endregion

		void context_BeginRequest(object sender, EventArgs e)
		{
		}

		
		void context_PreRequestHandlerExecute(object sender, EventArgs e)
		{
			HttpApplication app		= sender as HttpApplication;
			HttpContext context		= app.Context;

			#region Servervariables 값 보기 코드
			// 테스트용 코드
			//foreach (var item in context.Request.ServerVariables)
			//{
			//    context.Response.Write( item.ToString() + " = " + context.Request.ServerVariables[item.ToString()]+"<br/>");
			//} 
			#endregion
		}

		void context_PostRequestHandlerExecute(object sender, EventArgs e)
		{
			HttpApplication app		= sender as HttpApplication;
			HttpContext context		= app.Context;

			//if (app.Request["HTTP_X_MICROSOFTAJAX"] != null)
			//    return;

			// app.Context.Response.Cache.VaryByHeaders["Accept-Encoding"] = true;
			// TODO:context_PostRequestHandlerExecute - 응답 Content Type 를 받는다
			string content_Type		= context.Response.ContentType.Split(';')[0].Trim();

			// TODO:context_PostRequestHandlerExecute - 요청 경로를 받는다.
			string filename			= Path.GetFileName(context.Request.FilePath).ToLower();	// 요청파일명
			string path				= context.Request.Path.ToLower();
			

			UmcWebConfigSection.CoreWebHttpCompressionFileSection fileSection = UmcConfigManager.GetConfig().CoreWeb.HttpCompressionFile;

			// TODO:context_PostRequestHandlerExecute - 파일명이 Compressino Rule 에 존재하는지 검사
			if (fileSection[filename] != null && fileSection[filename].Enabled == true)
			{
			}
			else
			{
				// TODO:context_PostRequestHandlerExecute - File Rule 에 존재하지 않으면 RegexExpression 과 Path 를 뒤져 적절히 검사
				foreach (UmcWebConfigSection.CoreWebHttpCompressionFileElement element in fileSection)
				{
					HttpCompressionNameType nameType = (HttpCompressionNameType)Enum.Parse(typeof(HttpCompressionNameType), element.NameType);
					if (nameType == HttpCompressionNameType.RegexExpression)
					{
						if (Regex.IsMatch(filename, element.Name) && element.Enabled == false)
							return;
					}
					else if (nameType == HttpCompressionNameType.Path)
					{
						if( fileSection[path] != null && fileSection[path].Enabled == false )
							return;
					}
				}
			}
			
			// TODO:context_PostRequestHandlerExecute - 설정파일에 설정된 Content-Type 별로 압축을 수행할 조건
			UmcWebConfigSection.CoreWebHttpCompressionMimeElement mimeElement = UmcConfigManager.GetConfig().CoreWeb.HttpCompressionMime[content_Type];
			if( mimeElement != null && !mimeElement.Enabled) return;
			if( mimeElement == null ) return;

			// TODO:context_PostRequestHandlerExecute - 압축 알고리즘을 선택하여 압축을 수행한다
			HttpCompressionLevel level		= (HttpCompressionLevel)Enum.Parse(typeof(HttpCompressionLevel), mimeElement.Level);
			HttpCompressionType type		= (HttpCompressionType)Enum.Parse(typeof(HttpCompressionType), mimeElement.Type);

			// TODO:context_PostRequestHandlerExecute - 압축 알고리즘이 지정되지 않으면 압축을 수행하지 않는다.
			if (type != HttpCompressionType.None && type != HttpCompressionType.Custom)
			{
				HttpCompressionFactory.CreateFactory(type)
					.Compression(context, level);
			}
			else if (type == HttpCompressionType.Custom)
			{
				HttpCompressionFactory.CreateFactory(mimeElement.CustomProvider)
					.Compression(context, level);
			}
		}
	}
}
